#region usings
using System;
using System.ComponentModel.Composition;

using System.Xml.Linq;
using System.Xml.XPath;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	public abstract class XPathXMLChangeNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Element")]
		ISpread<XElement> FInput;

		[Input("XPath")]
		ISpread<string> FXPath;
		
		[Input("Content")]
		ISpread<XElement> FContent;

		[Output("Element")]
		ISpread<XElement> FOutput;

		[Import()]
		ILogger FLogger;
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			FOutput.SliceCount = SpreadMax;

			if(FInput.SliceCount > 0)
			for (int i = 0; i < SpreadMax; i++)
			{
				if(FInput[i] != null)
				{
					var xe = new XElement(FInput[i]);
					XElement k = xe.XPathSelectElement(FXPath[i]);
					if(k == null) {
						FOutput[i] = null;
						continue;
					}
					Do(k, FContent[i], i);
					FOutput[i] = xe;
				}
			}
		}
		
		public abstract void Do(XElement a, XElement b, int i);
	}
	
	
	#region PluginInfo
	[PluginInfo(Name = "ReplaceElements", Category = "XML", Version = "XPath", Help = "Basic template with one value in/out", Tags = "")]
	#endregion PluginInfo
	public class XPathXMLSetElementsNode : XPathXMLChangeNode
	{
		public override void Do(XElement a, XElement b, int i) {
			a.ReplaceWith(b);
		}
	}
	
	#region PluginInfo
	[PluginInfo(Name = "AddElements", Category = "XML", Version = "XPath", Help = "Basic template with one value in/out", Tags = "")]
	#endregion PluginInfo
	public class XPathXMLAddElementsNode : XPathXMLChangeNode
	{
		public enum Action {
			Add,
			AddAfterSelf,
			AddBeforeSelf
		}
		
		[Input("Action")]
		ISpread<Action> FAction;

		public override void Do(XElement a, XElement b,int i) {
			switch(FAction[i]){
				case Action.Add:
				a.Add(b);
				break;
				case Action.AddAfterSelf:
				a.AddAfterSelf(b);
				break;
				case Action.AddBeforeSelf:
				a.AddBeforeSelf(b);
				break;	
			}
		}
	}
	
	#region PluginInfo
	[PluginInfo(Name = "RemoveElements", Category = "XML", Version = "XPath", Help = "Basic template with one value in/out", Tags = "")]
	#endregion PluginInfo
	public class XPathXMLRemoveElementsNode : XPathXMLChangeNode
	{		
		public override void Do(XElement a, XElement b,int i) {
			a.Remove();
		}
	}
}
